package mateus.brum.chapter.one;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import brum.mateus.chapter.one.Customer;
import brum.mateus.chapter.one.Movie;
import brum.mateus.chapter.one.Rental;

public class CustomerTest {
	
	private static final String MOZARD = "Mozard";
	private static final String MEGA_MIND = "Mega Mind";
	private static final String TERMINATOR_SALVATION = "Terminator Salvation";
	private static final String TOTAL_RECALL = "Total Recall";
	Customer customer = null;
	Movie regularMovie = null;
	Movie newMovie = null;
	Movie childrenMovie = null;
	String ONE_RENTAL_PLAIN_TEXT = "Rental Record for %s\n\t%s\t%s\nAmount owed is %s\nYou earned %s frequent renter points";
	String THREE_RENTAL_PLAIN_TEXT = "Rental Record for %s\n\t%s\t%s\n\t%s\t%s\n\t%s\t%s\nAmount owed is %s\nYou earned %s frequent renter points";
	String THREE_RENTAL_HTML = "<H1>Rentals for <EM>%s</EM></H1><P>\n%s: %s<BR>\n%s: %s<BR>\n%s: %s<BR>\n<P>You owe <EM>%s</EM><P>\nOn this rental you earned <EM>%s</EM> frequent renter points<P>";

	
	@Before
	public void setup(){
		customer = new Customer(MOZARD);
		regularMovie = new Movie(TOTAL_RECALL, Movie.REGULAR);
		newMovie = new Movie(TERMINATOR_SALVATION, Movie.NEW_RELEASE);
		childrenMovie = new Movie(MEGA_MIND, Movie.CHILDRENS);
	}
	
	@Test
	public void testOneRegularMovie() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,TOTAL_RECALL,"2.0","2.0","1");
		Rental rental = new Rental(regularMovie,2);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testOneRegularMovieWithExcededDays() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,TOTAL_RECALL,"5.0","5.0","1");
		Rental rental = new Rental(regularMovie,4);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testOneChildrenMovie() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,MEGA_MIND,"1.5","1.5","1");
		Rental rental = new Rental(childrenMovie,3);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testOneChildrenMovieWithExcededDays() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,MEGA_MIND,"6.0","6.0","1");
		Rental rental = new Rental(childrenMovie,6);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testOneNewMovie() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,TERMINATOR_SALVATION,"9.0","9.0","2");
		Rental rental = new Rental(newMovie,3);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testOneNewMovieWithExcededDays() {
		String exp = String.format(ONE_RENTAL_PLAIN_TEXT,MOZARD,TERMINATOR_SALVATION,"27.0","27.0","2");
		Rental rental = new Rental(newMovie,9);
		customer.addRental(rental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testNewChildrenRegularMovies() {
		String exp = String.format(THREE_RENTAL_PLAIN_TEXT,MOZARD,TERMINATOR_SALVATION,"6.0",TOTAL_RECALL,"3.5",MEGA_MIND,"1.5","11.0","4");
		Rental firstRental = new Rental(newMovie,2);
		Rental secondRental = new Rental(regularMovie,3);
		Rental thirdRental = new Rental(childrenMovie,3);
		customer.addRental(firstRental);
		customer.addRental(secondRental);
		customer.addRental(thirdRental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testNewChildrenRegularMoviesWithExcededDays() {
		String exp = String.format(THREE_RENTAL_PLAIN_TEXT,MOZARD,TERMINATOR_SALVATION,"12.0",TOTAL_RECALL,"8.0",MEGA_MIND,"6.0","26.0","4");
		Rental firstRental = new Rental(newMovie,4);
		Rental secondRental = new Rental(regularMovie,6);
		Rental thirdRental = new Rental(childrenMovie,6);
		customer.addRental(firstRental);
		customer.addRental(secondRental);
		customer.addRental(thirdRental);
		Assert.assertEquals(exp, customer.statement());
	}
	
	@Test
	public void testNewChildrenRegularMoviesWithExcededDaysAsHTML() {
		String exp = String.format(THREE_RENTAL_HTML,MOZARD,TERMINATOR_SALVATION,"12.0",TOTAL_RECALL,"8.0",MEGA_MIND,"6.0","26.0","4");
		Rental firstRental = new Rental(newMovie,4);
		Rental secondRental = new Rental(regularMovie,6);
		Rental thirdRental = new Rental(childrenMovie,6);
		customer.addRental(firstRental);
		customer.addRental(secondRental);
		customer.addRental(thirdRental);
		Assert.assertEquals(exp, customer.htmlStatement());
	}
	
}
